import { expect, fixture, html } from '@open-wc/testing';
import './{{ tagWithoutPrefix tag }}.js';

describe('<{{ tag }}>', () => {
    it('should render a component', async () => {
        const el = await fixture(html` <{{ tag }}></{{ tag }}> `);

        expect(el).to.exist;
    });
});
